/***** extend.c ***************************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Sun Feb 12 19:42:03 2006.
 *****************************************************************/
#include <stdlib.h>
#include <string.h>
#include "interface.h"
#include "alignment.h"
#include "ktree.h"
#include "extend.h"
#include "eprintf.h"

int queryLen, sbjctLen;

Msp *checkDna(KTreeResult *ktr, char *query, char *sbjct, SubstitutionMatrix *sm, char *strand, Args args){
  KTreeResult *rp;
  ListElement *lp;
  float wordScore;
  Msp *msp, *mspList;

  msp = getMsp();;
  mspList = NULL;

  queryLen = strlen(query);
  sbjctLen = strlen(sbjct);

  msp->sbjctE = -1;
  wordScore = args.w*args.A;
  for(rp = ktr; rp != NULL; rp = rp->next){                    /* go through hits on sbjct */
    
    if(rp->pos > msp->sbjctE){                                 /* hit outside previous msp? */
      msp->sbjctS = rp->pos;
      msp->sbjctE = rp->pos + args.w - 1;
      msp->score = wordScore;
      for(lp=rp->data->tableEntry->posList; lp!=NULL; lp=lp->next){ /* go through list of matches in query */
	msp->queryS = lp->pos;
	msp->queryE = lp->pos + args.w - 1;
	extend(query, sbjct, msp, sm, args);                  /* extend matches */
	if(msp->score > args.t){ 
	  if(mspList == NULL || msp->sbjctE > mspList->sbjctE){
	     msp->next = mspList;                          /* add msp to list */
	     mspList = msp;
	     msp = getMsp();
	     msp->sbjctE = mspList->sbjctE;
	  }
	}else{
	  msp->sbjctE = rp->pos+args.w-1;
	  msp->sbjctS = rp->pos;
	}
	msp->score = wordScore;
	}      
    }
  }
  free(msp);
  return mspList;
}

  /*
   * Construct maximal segment pair 
   */
  Msp *getMsp(){
    Msp *msp;

    msp = (Msp *)emalloc(sizeof(Msp));
    msp->strand = NULL;
    return msp;
  }

  void extend(char *query, char *sbjct, Msp *msp, SubstitutionMatrix *sm, Args args){
    int maxScore;
    int maxIdleSteps;
    int idleSteps, debug, i;
  
    debug = 0;
    maxScore = msp->score;
    maxIdleSteps = args.s;
    idleSteps = 1;

    /* extend left */
    if(msp->sbjctS == 2615369 || msp->sbjctS == 2615394)
      debug = 1;
    if(debug){
      printf("before left: sbjct_start: %d, sbjct_end: %d\n",msp->sbjctS,msp->sbjctE);
      printf("before left: query_start: %d, query_end: %d\n",msp->queryS,msp->queryE);
    }
    if(debug){
      printf("->\n");
      for(i=0;i<20;i++)
	printf("%c",query[msp->queryE+i]);
      printf("\n");
      for(i=0;i<20;i++)
	printf("%c",sbjct[msp->sbjctE+i]);
      printf("\n");
      
      /*       printf("idle steps right: %d; score: %f\n",idleSteps, msp->score); */
    }
    if(debug){
      printf("<-\n");
      for(i=0;i<20;i++)
	printf("%c",query[msp->queryS-20+i]);
      printf("\n");
      for(i=0;i<20;i++)
	printf("%c",sbjct[msp->sbjctS-20+i]);
      printf("\n");
      
      /*       printf("idle steps right: %d; score: %f\n",idleSteps, msp->score); */
    }
    while(msp->queryS - idleSteps >= 0 && msp->sbjctS - idleSteps >= 0 && idleSteps < maxIdleSteps){
      msp->score += sm->mat[sm->dic[(int)query[msp->queryS-idleSteps]]][sm->dic[(int)sbjct[msp->sbjctS-idleSteps]]];
      if(msp->score > maxScore){
	maxScore = msp->score;
	msp->queryS -= idleSteps;
	msp->sbjctS -= idleSteps;
	idleSteps = 1;
      }else{
	idleSteps++;
      }
    }
    if(debug){
      printf("after left: sbjct_start: %d, sbjct_end: %d\n",msp->sbjctS,msp->sbjctE);
      printf("after left: query_start: %d, query_end: %d\n",msp->queryS,msp->queryE);
    }
    msp->score = maxScore;
    /* extend right */
    idleSteps = 1;
    while(msp->queryE+idleSteps < queryLen && msp->sbjctE+idleSteps < sbjctLen && idleSteps < maxIdleSteps){
      msp->score += sm->mat[sm->dic[(int)query[msp->queryE+idleSteps]]][sm->dic[(int)sbjct[msp->sbjctE+idleSteps]]];
      if(msp->score > maxScore){
	maxScore = msp->score;
	msp->queryE += idleSteps;
	msp->sbjctE += idleSteps;
	idleSteps = 1;
      }else{
	idleSteps++;
      }
    }
    if(debug){
      printf("after right: sbjct_start: %d, sbjct_end: %d\n",msp->sbjctS,msp->sbjctE);
      printf("after right: query_start: %d, query_end: %d\n",msp->queryS,msp->queryE);
      printf("******************************************\n");
    }
    msp->score = maxScore;
  }
