/***** hash.c *****************************************************
 * Description: Hashing of a string.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Fri Feb 10 05:42:40 2006.
 *****************************************************************/
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "eprintf.h"
#include "stringUtil.h"
#include "ktree.h"

static TableEntry *hashtab[HASHSIZE];

/* hash: form hash value for string s */
unsigned hash(char *s){
  unsigned hashval;

  for(hashval = 0; *s != '\0'; s++)
    hashval = *s + 31 * hashval;
  return hashval % HASHSIZE;
}

/* lookup look for s in hashtab */
TableEntry *lookup(char *s, int hashVal){
  TableEntry *tp;

  for(tp = hashtab[hashVal]; tp != NULL; tp = tp->next)
    if(strcmp(s, tp->word) == 0)
      return tp;     /* found */
  return NULL;       /* not found */
}

/* install: put (word, pos) in hashtab */
TableEntry *install(char *word, int pos){
  TableEntry *tp;
  ListElement *lp;
  unsigned hashval;

  lp = (ListElement *) emalloc(sizeof(*lp));
  lp->pos = pos;
  hashval = hash(word);
  if((tp = lookup(word,hashval)) == NULL) {   /* not found: make new table entry */
    tp = getTableEntry();
    if(tp == NULL || (tp->word = strdup2(word)) == NULL)
      return NULL;
    hashval = hash(word);
    tp->next = hashtab[hashval];
    hashtab[hashval] = tp;
  }                                 /* add position to list of positions */
  lp->next = tp->posList->next;
  tp->posList->next = lp;
  return tp;
}

TableEntry *getTableEntry(){
  TableEntry *tp;

   tp = (TableEntry *) emalloc(sizeof(*tp));
   tp->posList = (ListElement *) emalloc(sizeof(ListElement));

   return tp;
}

TableEntry **getHashTable(){
  return hashtab;
}
